/*
** Job Arranger for ZABBIX
** Copyright (C) 2012 FitechForce, Inc. All Rights Reserved.
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
**/

/*
** $Date:: 2012-12-03 16:25:38 +0900 #$
** $Revision: 3002 $
** $Author: kato@FITECHLABS.CO.JP $
**/

#include "common.h"

#include "threads.h"
#include "comms.h"
#include "cfg.h"
#include "log.h"
#include "zbxgetopt.h"
#include "zbxjson.h"

#include "jacommon.h"

#define JOBARG_DEFAULT_SERVER_PORT_STR	"10061"

/* long options */
static struct zbx_option	longopts[] =
{
	{"zabbix-server",	1,	NULL,	'z'},
	{"port",		1,	NULL,	'p'},
	{"host",		1,	NULL,	's'},
	{"source-address",	1,	NULL,	'I'},
	{"user-name",		1,	NULL,	'U'},
	{"password",		1,	NULL,	'P'},
	{"jobnet-id", 		1,	NULL,	'j'},
	{"help",		0,	NULL,	'h'},
	{NULL}
};

/* short options */
static char	shortopts[] = "I:z:p:s:U:P:j:h";

/* end of COMMAND LINE OPTIONS */

static int	CONFIG_LOG_LEVEL = LOG_LEVEL_WARNING;

static char	*JOBARG_SOURCE_IP = NULL;
static char	*JOBARG_SERVER = NULL;
unsigned short	JOBARG_SERVER_PORT = 0;
unsigned short	JOBARG_DEFAULT_SERVER_PORT = 10061;
static char	*JOBARG_HOSTNAME = NULL;
static char	*JOBARG_JOBNETID = NULL;
static char	*JOBARG_USERNAME = NULL;
static char	*JOBARG_PASSWORD = NULL;

const char	*progname = NULL;
const char	title_message[] = "Jobarranger Exec";
const char	usage_message[] = "[-h] -z <hostname or IP> [-p <port>] -s <hostname> -I <IP address> -U <username> -P <password> -j <jobnetid>";

const char	*help_message[] = {
	"Options:",
	"  -z --jobarranger-server <server>     Hostname or IP address of Job Arranger server",
	"  -p --port <server port>              Specify port number of server trapper running on the server. Default is " JOBARG_DEFAULT_SERVER_PORT_STR,
	"  -s --host <hostname>                 Specify host name. Host IP address and DNS name will not work",
	"  -I --source-address <IP address>     Specify source IP address",
	"  -U --user-name <user-name>           Specify user name",
	"  -P --password <password>             Specify password",
	"  -j --jobnet-id <jobnetid>            Specify jobnetid",
	"",
	"Other options:",
	"  -h --help                            Give this help",
	NULL	/* end of text */
};

/* COMMAND LINE OPTIONS */

#if !defined(_WINDOWS)

static void	send_signal_handler(int sig)
{
	if (SIGALRM == sig)
		zabbix_log(LOG_LEVEL_WARNING, "Timeout while executing operation");

	exit(FAIL);
}

#endif /* NOT _WINDOWS */

void	help_jobarg()
{
	const char	**p = help_message;

	usage();
	printf("\n");

	while (NULL != *p)
		printf("%s\n", *p++);
}

static int	check_response(char *response)
{
	struct zbx_json_parse	jp, jp_row;
	char			value[MAX_STRING_LEN];
	char			*message;
	const char		*p;
	int 			ret = FAIL;
	int			result;
	int			version;

	if ( SUCCEED == zbx_json_open(response, &jp))
	{
		if (SUCCEED == zbx_json_value_by_name(&jp, JA_PROTO_TAG_KIND, value, sizeof(value)))
		{
			if (0 != strcmp(value, JA_PROTO_VALUE_JOBNETRUN_RES)) {
				zabbix_log(LOG_LEVEL_ERR, "Received message error: [kind] is not [jobnetrun-res]");
				return ret;
			}
		} else {
			zabbix_log(LOG_LEVEL_ERR, "Received message error: [kind] not found");
			return ret;
		}

		if (SUCCEED == zbx_json_value_by_name(&jp, JA_PROTO_TAG_VERSION, value, sizeof(value)))
		{
			version = atoi(value);
			if ( version != JA_PROTO_VALUE_VERSION_1) {
				zabbix_log(LOG_LEVEL_ERR, "Received message error: [version] is not [%d]",JA_PROTO_VALUE_VERSION_1);
				return ret;
			}
		} else {
			zabbix_log(LOG_LEVEL_ERR, "Received message error: [version] not found");
			return ret;
		}

		if (NULL == (p = zbx_json_pair_by_name(&jp, JA_PROTO_TAG_DATA))) {
			zabbix_log(LOG_LEVEL_ERR, "Received message error: [data] not found");
			return ret;

		} else {
			if (FAIL == zbx_json_brackets_open(p, &jp_row)) {
				zabbix_log(LOG_LEVEL_ERR, "Received message error: Cannot open [data] object");
				return ret;

			} else {
				if (SUCCEED == zbx_json_value_by_name(&jp_row, JA_PROTO_TAG_RESULT, value, sizeof(value))) {
					result = atoi(value);
					if ( result == 0 ) {
						zabbix_log(LOG_LEVEL_DEBUG, "Received OK!!");
						ret =  SUCCEED;

					} else if ( result == 1 ) {
						
						if (SUCCEED == zbx_json_value_by_name(&jp_row, JA_PROTO_TAG_MESSAGE, value, sizeof(value))) {
							message = strdup(value);
							zabbix_log(LOG_LEVEL_ERR, "Cannnot send data: message [%s]", message);
						} else {
							zabbix_log(LOG_LEVEL_ERR, "Cannnot send data: cannnot get message from agent response");
							return ret;
						}

					} else {
						zabbix_log(LOG_LEVEL_ERR, "result range error : result [%d]"
							, result);
						return ret;
					}

				} else {
					zabbix_log(LOG_LEVEL_ERR, "Received message error: [result] not found");
					return ret;
				}
			}
		}
	} else {
		zabbix_log(LOG_LEVEL_ERR, "Cannot open received data.");
	}

	return ret;
}

static void	parse_commandline(int argc, char **argv)
{
	char	ch = '\0';

	/* parse the command-line */
	while ((char)EOF != (ch = (char)zbx_getopt_long(argc, argv, shortopts, longopts, NULL)))
	{
		switch (ch)
		{
			case 'h':
				help_jobarg();
				exit(-1);
				break;
			case 'I':
				JOBARG_SOURCE_IP = zbx_strdup(JOBARG_SOURCE_IP, zbx_optarg);
				break;
			case 'z':
				JOBARG_SERVER = zbx_strdup(JOBARG_SERVER, zbx_optarg);
				break;
			case 'p':
				JOBARG_SERVER_PORT = (unsigned short)atoi(zbx_optarg);
				break;
			case 's':
				JOBARG_HOSTNAME = zbx_strdup(JOBARG_HOSTNAME, zbx_optarg);
				break;
			case 'U':
				JOBARG_USERNAME = zbx_strdup(JOBARG_USERNAME, zbx_optarg);
				break;
			case 'P':
				JOBARG_PASSWORD = zbx_strdup(JOBARG_PASSWORD, zbx_optarg);
				break;
			case 'j':
				JOBARG_JOBNETID = zbx_strdup(JOBARG_JOBNETID, zbx_optarg);
				break;
			default:
				usage();
				exit(FAIL);
				break;
		}
	}

	if (NULL == JOBARG_SOURCE_IP || NULL == JOBARG_HOSTNAME || NULL == JOBARG_USERNAME || NULL == JOBARG_JOBNETID)
	{
		usage();
		exit(FAIL);
	}
}

int	main(int argc, char **argv)
{
	int			ret = SUCCEED;
	struct zbx_json 	json;
	int			tcp_ret;
	zbx_sock_t		sock;
	char			*answer = NULL;

	progname = get_program_name(argv[0]);

	parse_commandline(argc, argv);

	/*output to stderr*/
	zabbix_open_log(LOG_TYPE_UNDEFINED, CONFIG_LOG_LEVEL, NULL);

	if (NULL == JOBARG_SERVER)
	{
		zabbix_log(LOG_LEVEL_ERR, "'Server' parameter required");
		ret = FAIL;
		goto exit;
	}
	if (0 == JOBARG_SERVER_PORT)
	{
		JOBARG_SERVER_PORT = JOBARG_DEFAULT_SERVER_PORT;
	}
	if (MIN_ZABBIX_PORT > JOBARG_SERVER_PORT)
	{
		zabbix_log(LOG_LEVEL_ERR, "Incorrect port number [%d]. Allowed [%d:%d]",
				(int)JOBARG_SERVER_PORT, (int)MIN_ZABBIX_PORT, (int)MAX_ZABBIX_PORT);
		ret = FAIL;
		goto exit;
	}

	/*JSON data*/
	zbx_json_init(&json, ZBX_JSON_STAT_BUF_LEN);
	zbx_json_addstring(&json, JA_PROTO_TAG_KIND, JA_PROTO_VALUE_JOBNETRUN, ZBX_JSON_TYPE_STRING);
	zbx_json_adduint64(&json, JA_PROTO_TAG_VERSION, JA_PROTO_VALUE_VERSION_1);
	zbx_json_addobject(&json, JA_PROTO_TAG_DATA);
	zbx_json_addstring(&json, JA_PROTO_TAG_USERNAME, JOBARG_USERNAME, ZBX_JSON_TYPE_STRING);
	zbx_json_addstring(&json, JA_PROTO_TAG_PASSWORD, JOBARG_PASSWORD, ZBX_JSON_TYPE_STRING);
	zbx_json_addstring(&json, JA_PROTO_TAG_HOST, JOBARG_HOSTNAME, ZBX_JSON_TYPE_STRING);
	zbx_json_addstring(&json, JA_PROTO_TAG_JOBNETID, JOBARG_JOBNETID, ZBX_JSON_TYPE_STRING);
	zbx_json_close(&json);
	zbx_json_close(&json);

	zabbix_log(LOG_LEVEL_DEBUG, "JSON before sending [%s]", json.buffer);

#if !defined(_WINDOWS)
	signal(SIGINT,  send_signal_handler);
	signal(SIGTERM, send_signal_handler);
	signal(SIGQUIT, send_signal_handler);
	signal(SIGALRM, send_signal_handler);
#endif /* NOT _WINDOWS */

	if (SUCCEED == (tcp_ret = zbx_tcp_connect(&sock, JOBARG_SOURCE_IP, JOBARG_SERVER, JOBARG_SERVER_PORT, GET_SENDER_TIMEOUT)))
	{
		if (SUCCEED == (tcp_ret = zbx_tcp_send(&sock, json.buffer)))
		{
			if (SUCCEED == (tcp_ret = zbx_tcp_recv(&sock, &answer)))
			{
				if (NULL == answer || SUCCEED != check_response(answer)) {
					zabbix_log(LOG_LEVEL_DEBUG, "incorrect answer from server [%s]", answer);
					tcp_ret = FAIL;
				}
			}
		}
		zbx_tcp_close(&sock);
	} 

	if (FAIL == tcp_ret) {
		zabbix_log(LOG_LEVEL_ERR, "send value error: %s", zbx_tcp_strerror());
		ret = FAIL;
	}

exit:
	zabbix_close_log();

	return ret;
}
